"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const actions_1 = require("src/actions/");
const routeList_1 = require("src/routeList");
const util_1 = require("src/lib/util");
const sso_1 = require("../sso");
const ramda_1 = require("ramda");
const redux_1 = require("redux");
const typeorm_1 = require("typeorm");
const actionTypes_1 = require("./actionTypes");
const backendMiddleware_1 = require("src/backendMiddleware");
const checkRecoverySetup_1 = require("../notifications/checkRecoverySetup");
exports.setDid = (did) => ({
    type: 'DID_SET',
    value: did,
});
exports.setSelected = (claim) => ({
    type: 'SET_SELECTED',
    selected: claim,
});
exports.resetSelected = () => ({
    type: 'RESET_SELECTED',
});
exports.handleClaimInput = (fieldValue, fieldName) => ({
    type: 'HANDLE_CLAIM_INPUT',
    fieldName,
    fieldValue,
});
exports.checkIdentityExists = (dispatch, getState, backendMiddleware) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    try {
        const identityWallet = yield backendMiddleware.prepareIdentityWallet();
        const userDid = identityWallet.identity.did;
        dispatch(exports.setDid(userDid));
        return dispatch(actions_1.navigationActions.navigate({ routeName: routeList_1.routeList.Home }));
    }
    catch (err) {
        if (!(err instanceof backendMiddleware_1.BackendError))
            throw err;
        if (err.message === backendMiddleware_1.BackendError.codes.NoEntropy) {
            // No seed in database, user must register
            // But check if a registration was already in progress
            const isRegistering = getState().registration.loading.isRegistering;
            const routeName = isRegistering
                ? routeList_1.routeList.RegistrationProgress
                : routeList_1.routeList.Landing;
            return dispatch(actions_1.navigationActions.navigate({ routeName }));
        }
    }
});
exports.openClaimDetails = (claim) => dispatch => {
    dispatch(exports.setSelected(claim));
    return dispatch(actions_1.navigationActions.navigate({
        routeName: routeList_1.routeList.ClaimDetails,
    }));
};
exports.saveClaim = (dispatch, getState, backendMiddleware) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    const { identityWallet, storageLib, keyChainLib } = backendMiddleware;
    const did = getState().account.did.did;
    const claimsItem = getState().account.claims.selected;
    const password = yield keyChainLib.getPassword();
    const verifiableCredential = yield identityWallet.create.signedCredential({
        metadata: util_1.getClaimMetadataByCredentialType(claimsItem.credentialType),
        // the library acts directly on the object passed in, so a copy should be made first
        claim: Object.assign({}, claimsItem.claimData),
        subject: did,
    }, password);
    if (claimsItem.id) {
        yield storageLib.delete.verifiableCredential(claimsItem.id);
    }
    yield storageLib.store.verifiableCredential(verifiableCredential);
    yield dispatch(exports.setClaimsForDid);
    return dispatch(actions_1.navigationActions.navigatorResetHome());
});
// TODO Currently only rendering  / adding one
exports.saveExternalCredentials = (dispatch, getState, backendMiddleware) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    const { storageLib } = backendMiddleware;
    const externalCredentials = getState().account.claims.pendingExternal;
    if (!externalCredentials.offer.length) {
        return dispatch(sso_1.cancelReceiving);
    }
    const cred = externalCredentials.offer[0].credential;
    yield storageLib.delete.verifiableCredential(cred.id);
    yield storageLib.store.verifiableCredential(cred);
    yield dispatch(checkRecoverySetup_1.checkRecoverySetup);
    return dispatch(sso_1.cancelReceiving);
});
exports.toggleLoading = (value) => ({
    type: 'SET_LOADING',
    value,
});
exports.hasExternalCredentials = (dispatch, getState, backendMiddleware) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    const { storageLib, identityWallet } = backendMiddleware;
    const externalCredentials = yield storageLib.get.verifiableCredential({
        issuer: typeorm_1.Not(identityWallet.did),
    });
    return dispatch({
        type: actionTypes_1.HAS_EXTERNAL_CREDENTIALS,
        value: externalCredentials.length !== 0,
    });
});
exports.setClaimsForDid = (dispatch, getState, backendMiddleware) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    const { storageLib } = backendMiddleware;
    const verifiableCredentials = yield storageLib.get.verifiableCredential();
    const metadata = yield Promise.all(verifiableCredentials.map(el => storageLib.get.credentialMetadata(el)));
    const issuers = ramda_1.uniq(verifiableCredentials.map(cred => cred.issuer));
    const issuerMetadata = yield Promise.all(issuers.map(storageLib.get.publicProfile));
    const claims = exports.prepareClaimsForState(verifiableCredentials, metadata, issuerMetadata);
    return dispatch({
        type: 'SET_CLAIMS_FOR_DID',
        claims,
    });
});
exports.prepareClaimsForState = (credentials, credentialMetadata, issuerMetadata) => redux_1.compose(ramda_1.groupBy(util_1.getCredentialUiCategory), ramda_1.zipWith(ramda_1.mergeRight, credentialMetadata), ramda_1.map(exports.addIssuerInfo(issuerMetadata)), ramda_1.map(exports.convertToDecoratedClaim))(credentials);
exports.addIssuerInfo = (issuerProfiles) => (claim) => {
    if (!issuerProfiles || !issuerProfiles.length) {
        return claim;
    }
    const issuer = issuerProfiles.find(el => el.did === claim.issuer.did);
    return issuer
        ? Object.assign({}, claim, { issuer }) : claim;
};
/** @TODO Util, make subject mandatory (in lib) */
exports.convertToDecoratedClaim = ({ claim, type, issuer: issuerDid, id, expires, }) => ({
    credentialType: util_1.getUiCredentialTypeByType(type),
    issuer: {
        did: issuerDid,
    },
    claimData: ramda_1.omit(['id'], claim),
    id,
    subject: claim.id || 'Not found',
    expires: expires || undefined,
});
