"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const account_1 = require("./account");
const generic_1 = require("./generic");
/**
 * Curried function that wraps a {@link ThunkAction} with two calls to the provided loadingAction
 * {@link ActionCreator} (called with "true" before the wrapped action is dispatched, and with "false" after)
 * @param loadingAction - An action creator to create the actions to be dispatched before and after the wrappedAction-
 * @param wrappedAction - The thunkAction to be wrapped
 * @example dispatch(withLoading(toggleLoading)(saveClaims))
 */
exports.withLoadingHandler = (loadingAction) => (wrappedAction) => (dispatch) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    try {
        dispatch(loadingAction(true));
        return yield dispatch(wrappedAction);
    }
    finally {
        // NOTE: timeout is a hack to avoid flashing when navigating, as the new
        // screen will have not been loaded yet but the AppLoading will be taken
        // down, which will flash the old screen shortly
        setTimeout(() => dispatch(loadingAction(false)), 100);
    }
});
/**
 * Curried function that wraps a {@link ThunkAction} with the provided error handler to be dispatched on thrown error
 * @param errorHandler - An {@link ActionCreator} to be dispatched with the error if an error is thrown
 * @param errorModifier - An optional modifier for the error which is thrown, is Identity function by default
 * @param wrappedAction - The {@link ThunkAction} to be wrapped. If dispatching it throws an error, the errorhandler will be dispatched
 * @example dispatch(withErrorHandling(showErrorScreen)(saveClaims))
 * @dev The return value from this modifier can be passed to other modifiers, i.e. modifiers can be composed
 * e.g. dispatch(withLoading(toggleLoading)(withErrorHandling(showErrorScreen)(saveClaims)))
 */
exports.withErrorHandler = (errorHandler, errorModifier = undefined) => (wrappedAction, modifier = undefined) => (dispatch) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    modifier = modifier || errorModifier;
    try {
        return yield dispatch(wrappedAction);
    }
    catch (error) {
        if (modifier)
            error = modifier(error);
        return dispatch(errorHandler(error));
    }
});
exports.withLoading = exports.withLoadingHandler(account_1.toggleLoading);
exports.withErrorScreen = exports.withErrorHandler(generic_1.showErrorScreen);
