"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const react_native_1 = require("react-native");
const jolocom_lib_1 = require("jolocom-lib");
const actions_1 = require("src/actions");
const routeList_1 = require("src/routeList");
const util_1 = require("src/lib/util");
const account_1 = require("../account");
const helper_1 = require("jolocom-lib/js/utils/helper");
const errors_1 = require("src/lib/errors");
const utils_1 = require("./utils");
const node_fetch_1 = tslib_1.__importDefault(require("node-fetch"));
exports.setReceivingCredential = (requester, external) => ({
    type: 'SET_EXTERNAL',
    value: { offeror: requester, offer: external },
});
exports.receiveExternalCredential = (credReceive, offeror, isDeepLinkInteraction, credentialOfferMetadata) => (dispatch, getState, backendMiddleware) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    const { identityWallet, registry, storageLib } = backendMiddleware;
    yield identityWallet.validateJWT(credReceive, undefined, registry);
    const providedCredentials = credReceive.interactionToken.signedCredentials;
    const validationResults = yield jolocom_lib_1.JolocomLib.util.validateDigestables(providedCredentials);
    // TODO Error Code
    if (validationResults.includes(false)) {
        throw new Error('Invalid credentials received');
    }
    if (credentialOfferMetadata) {
        yield Promise.all(credentialOfferMetadata.map(storageLib.store.credentialMetadata));
    }
    if (offeror) {
        yield storageLib.store.issuerProfile(offeror);
    }
    // TODO change convertToDecoratedClaim to (metadata) => (cred): decoratedClaim
    // the types of the cred metadata arrays where it is use differ too much to do it simply right now
    const asDecoratedCredentials = providedCredentials.map(cred => {
        const md = credentialOfferMetadata
            ? credentialOfferMetadata.filter(mds => cred.type.includes(mds.type))
            : undefined;
        const renderInfo = md && md.length ? md[0].renderInfo : undefined;
        return Object.assign({}, account_1.convertToDecoratedClaim(cred), { renderInfo });
    });
    dispatch(exports.setReceivingCredential(offeror, providedCredentials.map((cred, i) => ({
        credential: cred,
        decoratedClaim: asDecoratedCredentials[i],
    }))));
    return dispatch(actions_1.navigationActions.navigate({
        routeName: routeList_1.routeList.CredentialDialog,
        params: { isDeepLinkInteraction },
        key: 'receiveExternalCredential',
    }));
});
exports.consumeCredentialRequest = (decodedCredentialRequest, isDeepLinkInteraction) => (dispatch, getState, backendMiddleware) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    const { storageLib, identityWallet, registry } = backendMiddleware;
    const { did } = getState().account.did;
    yield identityWallet.validateJWT(decodedCredentialRequest, undefined, registry);
    const requester = yield registry.resolve(helper_1.keyIdToDid(decodedCredentialRequest.issuer));
    const requesterSummary = utils_1.generateIdentitySummary(requester);
    const { requestedCredentialTypes: requestedTypes, } = decodedCredentialRequest.interactionToken;
    const attributesForType = yield Promise.all(requestedTypes.map(storageLib.get.attributesByType));
    const populatedWithCredentials = yield Promise.all(attributesForType.map((entry) => tslib_1.__awaiter(this, void 0, void 0, function* () {
        if (entry.results.length) {
            return Promise.all(entry.results.map((result) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                return ({
                    type: util_1.getUiCredentialTypeByType(entry.type),
                    values: result.values,
                    verifications: yield storageLib.get.verifiableCredential({
                        id: result.verification,
                    }),
                });
            })));
        }
        return [
            {
                type: util_1.getUiCredentialTypeByType(entry.type),
                values: [],
                verifications: [],
            },
        ];
    })));
    const abbreviated = populatedWithCredentials.map(attribute => attribute.map(entry => (Object.assign({}, entry, { verifications: entry.verifications.map((vCred) => ({
            id: vCred.id,
            issuer: {
                did: vCred.issuer,
            },
            selfSigned: vCred.signer.did === did,
            expires: vCred.expires,
        })) }))));
    const flattened = abbreviated.reduce((acc, val) => acc.concat(val));
    // TODO requester shouldn't be optional
    const credentialRequestDetails = {
        callbackURL: decodedCredentialRequest.interactionToken.callbackURL,
        requester: requesterSummary,
        availableCredentials: flattened,
        requestJWT: decodedCredentialRequest.encode(),
    };
    return dispatch(actions_1.navigationActions.navigate({
        routeName: routeList_1.routeList.Consent,
        params: { isDeepLinkInteraction, credentialRequestDetails },
        key: 'credentialRequest',
    }));
});
exports.sendCredentialResponse = (selectedCredentials, credentialRequestDetails, isDeepLinkInteraction = false) => (dispatch, getState, backendMiddleware) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    const { storageLib, keyChainLib, identityWallet } = backendMiddleware;
    const { callbackURL, requestJWT } = credentialRequestDetails;
    const password = yield keyChainLib.getPassword();
    const request = jolocom_lib_1.JolocomLib.parse.interactionToken.fromJWT(requestJWT);
    const credentials = yield Promise.all(selectedCredentials.map((cred) => tslib_1.__awaiter(this, void 0, void 0, function* () { return (yield storageLib.get.verifiableCredential({ id: cred.id }))[0]; })));
    const jsonCredentials = credentials.map(cred => cred.toJSON());
    const response = yield identityWallet.create.interactionTokens.response.share({
        callbackURL,
        suppliedCredentials: jsonCredentials,
    }, password, request);
    if (isDeepLinkInteraction) {
        const callback = `${callbackURL}${response.encode()}`;
        if (!(yield react_native_1.Linking.canOpenURL(callback))) {
            throw new errors_1.AppError(errors_1.ErrorCode.DeepLinkUrlNotFound);
        }
        return react_native_1.Linking.openURL(callback).then(() => dispatch(exports.cancelSSO));
    }
    yield node_fetch_1.default(callbackURL, {
        method: 'POST',
        body: JSON.stringify({ token: response.encode() }),
        headers: { 'Content-Type': 'application/json' },
    });
    dispatch(exports.cancelSSO);
});
exports.cancelSSO = dispatch => {
    return dispatch(actions_1.navigationActions.navigatorResetHome());
};
exports.cancelReceiving = dispatch => {
    dispatch(account_1.resetSelected());
    return dispatch(actions_1.navigationActions.navigatorResetHome());
};
