"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const identityWallet_1 = require("jolocom-lib/js/identityWallet/identityWallet");
const storage_1 = require("src/lib/storage/storage");
const keychain_1 = require("src/lib/keychain");
const jolocomRegistry_1 = require("jolocom-lib/js/registries/jolocomRegistry");
const ipfs_1 = require("src/lib/ipfs");
const contractsAdapter_1 = require("jolocom-lib/js/contracts/contractsAdapter");
const ethereum_1 = require("jolocom-lib/js/ethereum/ethereum");
const contractsGateway_1 = require("jolocom-lib/js/contracts/contractsGateway");
const jolocom_lib_1 = require("jolocom-lib");
const crypto_1 = require("jolocom-lib/js/utils/crypto");
const identity_1 = require("jolocom-lib/js/identity/identity");
const util_1 = require("src/lib/util");
var ErrorCodes;
(function (ErrorCodes) {
    ErrorCodes["NoEntropy"] = "NoEntropy";
    ErrorCodes["NoKeyProvider"] = "NoKeyProvider";
    ErrorCodes["NoWallet"] = "NoWallet";
    ErrorCodes["DecryptionFailed"] = "DecryptionFailed";
})(ErrorCodes = exports.ErrorCodes || (exports.ErrorCodes = {}));
class BackendError extends Error {
    constructor(code) {
        super(code);
    }
}
BackendError.codes = ErrorCodes;
exports.BackendError = BackendError;
class BackendMiddleware {
    constructor(config) {
        this.storageLib = new storage_1.Storage(config.typeOrmConfig);
        this.keyChainLib = new keychain_1.KeyChain();
        this.registry = jolocomRegistry_1.createJolocomRegistry({
            ipfsConnector: new ipfs_1.IpfsCustomConnector({
                host: 'ipfs.jolocom.com',
                port: 443,
                protocol: 'https',
            }),
            ethereumConnector: ethereum_1.jolocomEthereumResolver,
            contracts: {
                adapter: contractsAdapter_1.jolocomContractsAdapter,
                gateway: contractsGateway_1.jolocomContractsGateway,
            },
        });
    }
    initStorage() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield this.storageLib.initConnection();
        });
    }
    get identityWallet() {
        if (this._identityWallet)
            return this._identityWallet;
        throw new BackendError(ErrorCodes.NoWallet);
    }
    get keyProvider() {
        if (this._keyProvider)
            return this._keyProvider;
        throw new BackendError(ErrorCodes.NoKeyProvider);
    }
    prepareIdentityWallet() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (this._identityWallet)
                return this._identityWallet;
            const encryptedEntropy = yield this.storageLib.get.encryptedSeed();
            let encryptionPass;
            try {
                encryptionPass = yield this.keyChainLib.getPassword();
            }
            catch (e) {
                // This may fail if the application was uninstalled and reinstalled, as
                // the android keystore is cleared on uninstall, but the database may
                // still remain, due to having been auto backed up!
                // FIXME: Sentry.captureException(e)
            }
            if (encryptedEntropy && !encryptionPass) {
                // if we can't decrypt the encryptedEntropy, then reset the database
                console.warn('DROPPING OLD DB');
                yield this.storageLib.resetDatabase();
            }
            if (!encryptedEntropy || !encryptionPass) {
                // If either encryptedEntropy or encryptionPass was missing, we throw
                // NoEntropy to signal that we cannot prepare an identityWallet instance due
                // to lack of a seed.
                // Note that the case of having an encryptionPass but no encryptedEntropy
                // is an uncommon edge case, but may potentially happen due to errors/bugs
                // etc
                throw new BackendError(ErrorCodes.NoEntropy);
            }
            this._keyProvider = new jolocom_lib_1.JolocomLib.KeyProvider(Buffer.from(encryptedEntropy, 'hex'));
            const { jolocomIdentityKey: derivationPath } = jolocom_lib_1.JolocomLib.KeyTypes;
            const userPubKey = this._keyProvider.getPublicKey({
                derivationPath,
                encryptionPass,
            });
            const didDocument = yield this.storageLib.get.didDoc(crypto_1.publicKeyToDID(userPubKey));
            if (didDocument) {
                const identity = identity_1.Identity.fromDidDocument({ didDocument });
                // TODO Simplify constructor
                return (this._identityWallet = new identityWallet_1.IdentityWallet({
                    identity,
                    vaultedKeyProvider: this._keyProvider,
                    publicKeyMetadata: {
                        derivationPath,
                        keyId: identity.publicKeySection[0].id,
                    },
                    contractsAdapter: this.registry.contractsAdapter,
                    contractsGateway: this.registry.contractsGateway,
                }));
            }
            else {
                const identityWallet = yield this.registry.authenticate(this._keyProvider, {
                    encryptionPass,
                    derivationPath,
                });
                yield this.storageLib.store.didDoc(identityWallet.didDocument);
                return (this._identityWallet = identityWallet);
            }
        });
    }
    recoverIdentity(mnemonic) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const password = (yield util_1.generateSecureRandomBytes(32)).toString('base64');
            this._keyProvider = jolocom_lib_1.JolocomLib.KeyProvider.recoverKeyPair(mnemonic, password);
            const { jolocomIdentityKey: derivationPath } = jolocom_lib_1.JolocomLib.KeyTypes;
            const identityWallet = yield this.registry.authenticate(this._keyProvider, {
                encryptionPass: password,
                derivationPath,
            });
            this._identityWallet = identityWallet;
            yield this.keyChainLib.savePassword(password);
            yield this.storeIdentityData();
            return identityWallet.identity;
        });
    }
    createKeyProvider(encodedEntropy) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const password = (yield util_1.generateSecureRandomBytes(32)).toString('base64');
            this._keyProvider = jolocom_lib_1.JolocomLib.KeyProvider.fromSeed(Buffer.from(encodedEntropy, 'hex'), password);
            yield this.keyChainLib.savePassword(password);
        });
    }
    fuelKeyWithEther() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const password = yield this.keyChainLib.getPassword();
            yield jolocom_lib_1.JolocomLib.util.fuelKeyWithEther(this.keyProvider.getPublicKey({
                encryptionPass: password,
                derivationPath: jolocom_lib_1.JolocomLib.KeyTypes.ethereumKey,
            }));
        });
    }
    createIdentity() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const password = yield this.keyChainLib.getPassword();
            this._identityWallet = yield this.registry.create(this.keyProvider, password);
            yield this.storeIdentityData();
            return this._identityWallet.identity;
        });
    }
    storeIdentityData() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const personaData = {
                did: this._identityWallet.identity.did,
                controllingKeyPath: jolocom_lib_1.JolocomLib.KeyTypes.jolocomIdentityKey,
            };
            yield this.storageLib.store.persona(personaData);
            const encryptedSeedData = {
                // TODO: change to keyProvider.encryptedSeed when the library is updated
                // with a public getter for the encryptedSeed
                encryptedEntropy: this.keyProvider['encryptedSeed'].toString('hex'),
                timestamp: Date.now(),
            };
            yield this.storageLib.store.encryptedSeed(encryptedSeedData);
            yield this.storageLib.store.didDoc(this._identityWallet.didDocument);
        });
    }
}
exports.BackendMiddleware = BackendMiddleware;
