"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const node_polyfill_1 = require("src/node.polyfill");
exports.initStore = node_polyfill_1.initStore;
const entities = tslib_1.__importStar(require("src/lib/storage/entities"));
exports.entities = entities;
const actions = tslib_1.__importStar(require("src/actions"));
exports.actions = actions;
const jolocom_lib_1 = require("jolocom-lib");
const interactionTokens_1 = require("src/lib/storage/interactionTokens");
const modifiers_1 = require("./actions/modifiers");
const initErrorHandler = (error) => dispatch => {
    console.error(error.message);
    return Promise.reject(error);
};
class JolocomSDK {
    constructor(store) {
        this.store = store;
        this.dispatch = this.store.dispatch;
        this.idw = this.store.backendMiddleware.identityWallet;
    }
    /**
     * Returns an agent with an Identity provided by a store
     *
     * @remarks
     * This depends on an existing database connection
     *
     * @param store - The agent state with storage connection
     * @returns An Agent with the identity existing in the storage in store
     */
    static fromStore(store) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield store.dispatch(modifiers_1.withErrorHandler(initErrorHandler)(actions.accountActions.checkIdentityExists));
            return new JolocomSDK(store);
        });
    }
    /**
     * Returns an agent with an Identity provided by a BIP 39 12 word seed phrase
     *
     * @param mnemonic - 12 word BIP 39 seed phrase, space-delimited
     * @returns An Agent with the identity corrosponding to the sead phrase
     */
    static fromMnemonic(mnemonic) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const store = node_polyfill_1.initStore();
            yield store.dispatch(actions.registrationActions.recoverIdentity(mnemonic));
            return new JolocomSDK(store);
        });
    }
    /**
     * Returns an agent with an Identity provided by a BIP 39 12 word seed phrase
     *
     * @param mnemonic - 12 word BIP 39 seed phrase, space-delimited
     * @returns An Agent with the identity corrosponding to the sead phrase
     */
    static newDIDFromSeed(seed) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const store = node_polyfill_1.initStore();
            yield store.dispatch(actions.registrationActions.createIdentity(seed.toString('hex')));
            return new JolocomSDK(store);
        });
    }
    /**
     * Returns an agent with an Identity provided by a BIP 39 12 word seed phrase
     *
     * @param mnemonic - 12 word BIP 39 seed phrase, space-delimited
     * @returns An Agent with the identity corrosponding to the sead phrase
     */
    tokenRecieved(jwt) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const token = jolocom_lib_1.JolocomLib.parse.interactionToken.fromJWT(jwt);
            yield this.store.backendMiddleware.storageLib.store.interactionToken(token);
            yield this.dispatch(interactionTokens_1.interactionHandlers[token.interactionType](token));
        });
    }
    /**
     * Returns an agent with an Identity provided by a BIP 39 12 word seed phrase
     *
     * @param mnemonic - 12 word BIP 39 seed phrase, space-delimited
     * @returns An Agent with the identity corrosponding to the sead phrase
     */
    credRequestToken(request) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const token = yield this.idw.create.interactionTokens.request.share(request, yield this.store.backendMiddleware.keyChainLib.getPassword());
            this.store.backendMiddleware.storageLib.store.interactionToken(token);
            return token.encode();
        });
    }
    /**
     * Returns a base64 encoded signed credential offer token, given
     * request attributes
     *
     * @param offer - credential offer attributes
     * @returns A base64 encoded signed credential offer token offering
     * credentials according to `offer`
     */
    credOfferToken(offer) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const token = yield this.idw.create.interactionTokens.request.offer(offer, yield this.store.backendMiddleware.keyChainLib.getPassword());
            this.store.backendMiddleware.storageLib.store.interactionToken(token);
            return token.encode();
        });
    }
    /**
     * Returns a base64 encoded signed credential issuance token, given
     * issuance attributes and a recieved token selecting desired issuance
     *
     * @param issuance - credential issuance attributes
     * @param selection - base64 encoded credential offer response token
     * @returns A base64 encoded signed issuance token containing verifiable
     * credentials
     */
    credIssuanceToken(issuance, selection) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const token = yield this.idw.create.interactionTokens.response.issue(issuance, yield this.store.backendMiddleware.keyChainLib.getPassword(), yield jolocom_lib_1.JolocomLib.parse.interactionToken.fromJWT(selection));
            this.store.backendMiddleware.storageLib.store.interactionToken(token);
            return token.encode();
        });
    }
}
exports.JolocomSDK = JolocomSDK;
