"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const browser_1 = require("typeorm/browser");
const class_transformer_1 = require("class-transformer");
const entities_1 = require("src/lib/storage/entities");
const didDocument_1 = require("jolocom-lib/js/identity/didDocument/didDocument");
const utils_1 = require("./utils");
const migration_1 = tslib_1.__importDefault(require("./migration"));
const jolocom_lib_1 = require("jolocom-lib");
class Storage {
    constructor(config) {
        this.store = {
            setting: this.saveSetting.bind(this),
            persona: this.storePersonaFromJSON.bind(this),
            verifiableCredential: this.storeVClaim.bind(this),
            encryptedSeed: this.storeEncryptedSeed.bind(this),
            credentialMetadata: (metadata) => this.createConnectionIfNeeded().then(() => storeCredentialMetadata(this.connection)(metadata)),
            issuerProfile: (issuer) => this.createConnectionIfNeeded().then(() => storeIssuerProfile(this.connection)(issuer)),
            didDoc: (doc) => this.createConnectionIfNeeded().then(() => cacheDIDDoc(this.connection)(doc)),
            interactionToken: (token) => this.createConnectionIfNeeded().then(() => storeInteractionToken(this.connection)(token)),
        };
        this.get = {
            settingsObject: this.getSettingsObject.bind(this),
            setting: this.getSetting.bind(this),
            persona: this.getPersonas.bind(this),
            verifiableCredential: this.getVCredential.bind(this),
            attributesByType: this.getAttributesByType.bind(this),
            vCredentialsByAttributeValue: this.getVCredentialsForAttribute.bind(this),
            encryptedSeed: this.getEncryptedSeed.bind(this),
            credentialMetadata: (credential) => this.createConnectionIfNeeded().then(() => getMetadataForCredential(this.connection)(credential)),
            publicProfile: (did) => this.createConnectionIfNeeded().then(() => getPublicProfile(this.connection)(did)),
            didDoc: (did) => this.createConnectionIfNeeded().then(() => getCachedDIDDoc(this.connection)(did)),
            interactionTokens: (nonce) => this.createConnectionIfNeeded().then(() => getTokensByNonce(this.connection)(nonce)),
        };
        this.delete = {
            verifiableCredential: this.deleteVCred.bind(this),
        };
        this.initConnection = this.createConnectionIfNeeded.bind(this);
        this.config = Object.assign({}, config, { entities: entities_1.entityList, migrations: migration_1.default });
    }
    resetDatabase() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield this.createConnectionIfNeeded();
            yield this.connection.dropDatabase();
            yield this.connection.close();
            this.connectionPromise = null;
            yield this.createConnectionIfNeeded();
        });
    }
    createConnectionIfNeeded() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (this.connectionPromise)
                return this.connectionPromise;
            return (this.connectionPromise = browser_1.createConnection(this.config)
                .then(conn => {
                this.connection = conn;
                return conn;
            })
                .catch(err => {
                this.connectionPromise = null;
                throw err;
            }));
        });
    }
    getSettingsObject() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield this.createConnectionIfNeeded();
            const settingsList = yield this.connection.manager.find(entities_1.SettingEntity);
            const settings = {};
            settingsList.forEach(setting => {
                settings[setting.key] = setting.value;
            });
            return settings;
        });
    }
    getSetting(key) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield this.createConnectionIfNeeded();
            const setting = yield this.connection.manager.findOne(entities_1.SettingEntity, {
                key,
            });
            if (setting)
                return setting.value;
        });
    }
    saveSetting(key, value) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield this.createConnectionIfNeeded();
            const repo = this.connection.getRepository(entities_1.SettingEntity);
            const setting = repo.create({ key, value });
            yield repo.save(setting);
        });
    }
    // TODO: refactor needed on multiple personas
    getPersonas(query) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield this.createConnectionIfNeeded();
            return this.connection.manager.find(entities_1.PersonaEntity);
        });
    }
    getVCredential(query) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield this.createConnectionIfNeeded();
            const entities = yield this.connection.manager.find(entities_1.VerifiableCredentialEntity, {
                where: query,
                relations: ['claim', 'proof', 'subject'],
            });
            return entities.map(e => e.toVerifiableCredential());
        });
    }
    getAttributesByType(type) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield this.createConnectionIfNeeded();
            const localAttributes = yield this.connection
                .getRepository(entities_1.CredentialEntity)
                .createQueryBuilder('credential')
                .leftJoinAndSelect('credential.verifiableCredential', 'verifiableCredential')
                .where('verifiableCredential.type = :type', { type })
                .getMany();
            const results = utils_1.groupAttributesByCredentialId(localAttributes).map(entry => ({
                verification: entry.verifiableCredential.id,
                values: entry.propertyValue,
                fieldName: entry.propertyName,
            }));
            return { type, results };
        });
    }
    getVCredentialsForAttribute(attribute) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield this.createConnectionIfNeeded();
            const entities = yield this.connection
                .getRepository(entities_1.VerifiableCredentialEntity)
                .createQueryBuilder('verifiableCredential')
                .leftJoinAndSelect('verifiableCredential.claim', 'claim')
                .leftJoinAndSelect('verifiableCredential.proof', 'proof')
                .leftJoinAndSelect('verifiableCredential.subject', 'subject')
                .where('claim.propertyValue = :attribute', { attribute })
                .getMany();
            return entities.map(e => e.toVerifiableCredential());
        });
    }
    getEncryptedSeed() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield this.createConnectionIfNeeded();
            const masterKeyEntity = yield this.connection.manager.find(entities_1.MasterKeyEntity);
            if (masterKeyEntity.length) {
                return masterKeyEntity[0].encryptedEntropy;
            }
            return null;
        });
    }
    storePersonaFromJSON(args) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield this.createConnectionIfNeeded();
            const persona = class_transformer_1.plainToClass(entities_1.PersonaEntity, args);
            yield this.connection.manager.save(persona);
        });
    }
    storeEncryptedSeed(args) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield this.createConnectionIfNeeded();
            const encryptedSeed = class_transformer_1.plainToClass(entities_1.MasterKeyEntity, args);
            yield this.connection.manager.save(encryptedSeed);
        });
    }
    storeVClaim(vCred) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield this.createConnectionIfNeeded();
            const verifiableCredential = entities_1.VerifiableCredentialEntity.fromVerifiableCredential(vCred);
            const signature = entities_1.SignatureEntity.fromLinkedDataSignature(vCred.proof);
            const claims = entities_1.CredentialEntity.fromVerifiableCredential(vCred);
            claims.forEach(claim => (claim.verifiableCredential = verifiableCredential));
            signature.verifiableCredential = verifiableCredential;
            verifiableCredential.proof = [signature];
            verifiableCredential.claim = claims;
            yield this.connection.manager.save(verifiableCredential);
        });
    }
    deleteVCred(id) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield this.createConnectionIfNeeded();
            yield this.connection.manager
                .createQueryBuilder()
                .delete()
                .from(entities_1.CredentialEntity)
                .where('verifiableCredential = :id', { id })
                .execute();
            yield this.connection.manager
                .createQueryBuilder()
                .delete()
                .from(entities_1.SignatureEntity)
                .where('verifiableCredential = :id', { id })
                .delete()
                .execute();
            yield this.connection.manager
                .createQueryBuilder()
                .delete()
                .from(entities_1.VerifiableCredentialEntity)
                .where('id = :id', { id })
                .execute();
        });
    }
}
exports.Storage = Storage;
const cacheDIDDoc = (connection) => (doc) => {
    const cacheEntry = class_transformer_1.plainToClass(entities_1.CacheEntity, {
        key: `didCache:${doc.did}`,
        value: doc.toJSON(),
    });
    return connection.manager.save(cacheEntry);
};
const getCachedDIDDoc = (connection) => (did) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    const [entry] = yield connection.manager.findByIds(entities_1.CacheEntity, [
        `didCache:${did}`,
    ]);
    try {
        return didDocument_1.DidDocument.fromJSON(entry.value);
    }
    catch (err) {
        return undefined;
    }
});
const storeInteractionToken = (connection) => (token) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    const tokenEntry = entities_1.InteractionTokenEntity.fromJWT(token);
    return connection.manager.save(tokenEntry);
});
const getTokensByNonce = (connection) => (nonce) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    // return await connection.manager.find(InteractionTokenEntity)
    return connection.manager
        .find(entities_1.InteractionTokenEntity, {
        where: [{ nonce }],
    })
        .then(entities => entities.map(entity => jolocom_lib_1.JolocomLib.parse.interactionToken.fromJWT(entity.original)));
});
const storeCredentialMetadata = (connection) => (credentialMetadata) => {
    const { issuer, type: credentialType } = credentialMetadata;
    const cacheEntry = class_transformer_1.plainToClass(entities_1.CacheEntity, {
        key: buildMetadataKey(issuer.did, credentialType),
        value: Object.assign({}, credentialMetadata, { issuer: credentialMetadata.issuer.did }),
    });
    return connection.manager.save(cacheEntry);
};
const getMetadataForCredential = (connection) => ({ issuer, type: credentialType, }) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    const entryKey = buildMetadataKey(issuer, credentialType);
    const [entry] = yield connection.manager.findByIds(entities_1.CacheEntity, [entryKey]);
    return (entry && entry.value) || {};
});
const buildMetadataKey = (issuer, credentialType) => {
    if (typeof credentialType === 'string') {
        return `${issuer}${credentialType}`;
    }
    return `${issuer}${credentialType[credentialType.length - 1]}`;
};
const storeIssuerProfile = (connection) => (issuer) => {
    const cacheEntry = class_transformer_1.plainToClass(entities_1.CacheEntity, {
        key: issuer.did,
        value: issuer,
    });
    return connection.manager.save(cacheEntry);
};
const getPublicProfile = (connection) => (did) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    const [issuerProfile] = yield connection.manager.findByIds(entities_1.CacheEntity, [did]);
    return (issuerProfile && issuerProfile.value) || { did };
});
